package com.nsgk.ruralWeb.ui;

import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.Html;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.preference.Preference;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.PreferenceManager;
import androidx.preference.SeekBarPreference;

import com.nsgk.ruralWeb.BuildConfig;
import com.nsgk.ruralWeb.R;
import com.nsgk.ruralWeb.sys.NSConstants;
import com.nsgk.ruralWeb.sys.NSPreference;
import com.nsgk.ruralWeb.utils.NSContextUtils;
import com.nsgk.ruralWeb.utils.NSMisc;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;

public class SettingsFragment extends PreferenceFragmentCompat implements Preference.OnPreferenceChangeListener, Preference.OnPreferenceClickListener
{
    private static final String ID_TAG = SettingsFragment.class.getName();

    @Override
    public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
        addPreferencesFromResource(R.xml.settings_preference);
        Preference preference;
        final Context context = getContext();

        preference = findPreference(NSPreference.LOCATION_SCHEME);
        preference.setDefaultValue(NSConstants.DEFAULT_LOCATION_SCHEME);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_GAODE_INTERVAL);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_GAODE_READ_COUNT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.OPEN_LAST_URL);
        preference.setDefaultValue(NSConstants.DEFAULT_OPEN_LAST_URL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_PROVIDER);
        preference.setDefaultValue(NSConstants.DEFAULT_LOCATION_PROVIDER);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_INTERVAL);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_DISTANCE);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_READ_COUNT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.FONT_SCALE);
        preference.setDefaultValue(NSConstants.DEFAULT_FONT_SCALE);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.RESET_SETTINGS);
        preference.setOnPreferenceClickListener(this);

        preference = findPreference(NSPreference.VERSION);
        preference.setOnPreferenceClickListener(this);
    }

    private boolean CheckValueIsNumber(Object newValue, Integer min)
    {
        if(null != newValue)
        {
            String str = newValue.toString();
            if(StrUtil.isNotBlank(str))
            {
                if(!NumberUtil.isInteger(str))
                {
                    Toast.makeText(getContext(), "请输入数字", Toast.LENGTH_SHORT).show();
                    return false;
                }
                int i = Integer.parseInt(str);
                if(null != min)
                {
                    if(i < min)
                    {
                        Toast.makeText(getContext(), "必须大于等于" + min, Toast.LENGTH_SHORT).show();
                        return false;
                    }
                }
            }
        }
        return true;
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object newValue) {
        String key = preference.getKey();

        switch(key)
        {
            case NSPreference.LOCATION_GAODE_INTERVAL:
                if(!CheckValueIsNumber(newValue, 1000))
                    return false;
                break;
            case NSPreference.LOCATION_GAODE_READ_COUNT:
                if(!CheckValueIsNumber(newValue, 1))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_INTERVAL:
                if(!CheckValueIsNumber(newValue, 1000))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_DISTANCE:
                if(!CheckValueIsNumber(newValue, 0))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_READ_COUNT:
                if(!CheckValueIsNumber(newValue, 1))
                    return false;
                break;
            case NSPreference.FONT_SCALE: {
                int i = (int)newValue;
                int newi = Math.round((float)i / 10.0f) * 10;
                if(i < 50)
                    i = 50;
                if(i != newi)
                {
                    getView().post(() -> {
                        ((SeekBarPreference)preference).setValue(newi);
                        preference.callChangeListener(newi);
                    });
                    return false;
                }
                break;
            }
        }

        SetSummary(key, newValue);
        return true;
    }

    @Override
    public boolean onPreferenceClick(final Preference preference) {
        String key = preference.getKey();
        final Context context = getContext();

        switch(key)
        {
            case NSPreference.RESET_SETTINGS:
                OpenQueryDialog("警告", "确定要重置到默认设置?", (dialog, which) -> {
                    ResetSettings();
                    SetSummary(null, null);
                    Toast.makeText(context, "设置已重置", Toast.LENGTH_LONG).show();
                });
                break;
            case NSPreference.VERSION:
                OpenAbout();
                break;
        }
        return false;
    }

    private void ResetSettings()
    {
        NSPreference preference = new NSPreference(getContext());
        preference.Write()
                .putString(NSPreference.LOCATION_SCHEME, NSConstants.DEFAULT_LOCATION_SCHEME)
                .putString(NSPreference.LOCATION_GAODE_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL)
                .putString(NSPreference.LOCATION_GAODE_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT)
                .putBoolean(NSPreference.OPEN_LAST_URL, NSConstants.DEFAULT_OPEN_LAST_URL)
                .putString(NSPreference.LOCATION_PROVIDER, NSConstants.DEFAULT_LOCATION_PROVIDER)
                .putString(NSPreference.LOCATION_REALTIME_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL)
                .putString(NSPreference.LOCATION_REALTIME_DISTANCE, "" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE)
                .putString(NSPreference.LOCATION_REALTIME_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT)
                .putInt(NSPreference.FONT_SCALE, NSConstants.DEFAULT_FONT_SCALE)
                .commit()
        ;
    }

    private void OpenQueryDialog(String title, String message, DialogInterface.OnClickListener listener)
    {
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setIcon(R.drawable.no_words);
        builder.setPositiveButton("确定", listener);
        builder.setNegativeButton("取消", null);
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    private void OpenAbout()
    {
        Context context = getContext();
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        String msg = "<br/>" + NSContextUtils.tr(context, R.string.company) + "<br/><br/>" + NSContextUtils.tr(context, R.string.copyright);
        msg = "<div style='text-align:center;'>" + msg + "</div>";
        builder.setTitle(R.string.app_name2);
        builder.setMessage(Html.fromHtml(msg, Html.FROM_HTML_MODE_LEGACY, null, null));
        int icon = context.getResources().getIdentifier("ic_launcher_" + NSConstants.AppIcon()/* + "_round"*/, "mipmap", context.getApplicationContext().getPackageName());
        builder.setIcon(icon);
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    @Override
    public void onResume() {
        super.onResume();
        SetSummary(null, null);
    }

    public void SetSummary(String key, Object newValue)
    {
        Context context = getContext();
        Preference preference;
        String summary;
        String value;
        boolean b;
        int i;
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(context);

        if(key == null || NSPreference.LOCATION_SCHEME.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_SCHEME);
            value = newValue != null ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_SCHEME, NSConstants.DEFAULT_LOCATION_SCHEME);
            summary = NSContextUtils.GetListName(context, value, R.array.location_scheme_values, R.array.location_scheme_labels);
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_GAODE_INTERVAL.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_GAODE_INTERVAL);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_GAODE_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL);
            summary = value + "毫秒";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_GAODE_READ_COUNT.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_GAODE_READ_COUNT);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_GAODE_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT);
            summary = value + "次";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.OPEN_LAST_URL.equals(key))
        {
            preference = findPreference(NSPreference.OPEN_LAST_URL);
            b = newValue != null ? (Boolean)newValue : sharedPreferences.getBoolean(NSPreference.OPEN_LAST_URL, NSConstants.DEFAULT_OPEN_LAST_URL);
            summary = b ? "打开最近页面" : "打开默认页面";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_PROVIDER.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_PROVIDER);
            value = newValue != null ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_PROVIDER, NSConstants.DEFAULT_LOCATION_PROVIDER);
            summary = NSContextUtils.GetListName(context, value, R.array.location_provider_values, R.array.location_provider_labels);
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_INTERVAL.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_INTERVAL);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL);
            summary = value + "毫秒";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_DISTANCE.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_DISTANCE);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_DISTANCE, "" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE);
            summary = value + "米";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_READ_COUNT.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_READ_COUNT);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT);
            summary = value + "次";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.FONT_SCALE.equals(key))
        {
            preference = findPreference(NSPreference.FONT_SCALE);
            i = newValue != null ? (int)newValue : sharedPreferences.getInt(NSPreference.FONT_SCALE, NSConstants.DEFAULT_FONT_SCALE);
            if(i == 0)
                i = 100;
            summary = i + "%";
            preference.setSummary(summary);
        }

        findPreference(NSPreference.VERSION).setSummary(NSContextUtils.GetAppVersion(getContext()));
    }
}

